/*
 * e4-o.1.c
 * Copyright (C) 2007-2012, 2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <string.h>

#include <e4-types.h>

extern const char *_x1f4_e4_type_names[];

static void print_type(FILE *, int);

static void
print_type(FILE *file, int type)
{
    if (type < SUIT) {
	unsigned i, j;

	j = strlen(_x1f4_e4_type_names[type]);
	if (j < 16) {
	    j = 16 - j;
	} else {
	    j = 0;
	}
	i = j >> 1;
	j -= i;
	for (; i; i--) {
	    fputc(' ', file);
	}
	fputs(_x1f4_e4_type_names[type], file);
	for (; j; j--) {
	    fputc(' ', file);
	}
    } else {
	fputs("     <user>     ", file);
    }
}


void
x1f4_print_boperators(FILE *file, const struct e4_odbx_type *const *list,
		      int head)
{
    const struct e4_odbx_type *odbx_data;

    if (head) {
	fputs("      Type        Left Operand           Right Operand      "
	      "     Precedence\n", file);
    }

    odbx_data = *list;
    while (odbx_data) {
	const int *args;
	unsigned i, j;

	print_type(file, odbx_data->type);
	args = odbx_data->args;
	print_type(file, *args);
	j = 8 - odbx_data->size;
	i = j >> 1;
	j -= i;
	for (; i; i--) {
	    fputc(' ', file);
	}
	fputs(odbx_data->name, file);
	for (; j; j--) {
	    fputc(' ', file);
	}
	args++;
	print_type(file, *args);
	j = 8;
	if (odbx_data->bits & E2ND_LINK) {
	    const struct e4_2nde_type *e2nd_data;

	    e2nd_data = odbx_data->extension2;
	    if (e2nd_data->bits & POST_MARK) {
		const struct e4_mark_type *mark_data;

		mark_data = e2nd_data->post;
		j -= mark_data->size;
		i = j >> 1;
		j -= i;
		for (; i; i--) {
		    fputc(' ', file);
		}
		fputs(mark_data->part, file);
	    }
	}
	for (; j; j--) {
	    fputc(' ', file);
	}
	fprintf(file, "    %#4o\n", odbx_data->priority);
	list++;
	odbx_data = *list;
    }
}
