/*
 * e4.7.3.c
 * Copyright (C) 2006-2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <e4-config.h>

#if defined HAVE_LIBx1f4i0
# include <libx1f4i0.h>
#endif				/* HAVE_LIBx1f4i0 */
#include <stddef.h>
#include <string.h>

#include <e4-defs.h>
#include <e4-inter.h>
#include <e4-parse.h>
#include <e4-types.h>

#define M_FREE(mcontext, mdata) \
    (mcontext)->m.free((mcontext)->m.data, (mdata))
#define M_LINK(mcontext, mdata, size) \
    (mcontext)->m.link((mcontext)->m.data, (void *) (mdata), (size))
#define M_MODE(mcontext, mdata, size) \
    (mcontext)->m.mode((mcontext)->m.data, (void *) (mdata), (size))

static int lend_atom(struct e4_atom_type **, unsigned *, int *, int,
		     struct parser_type *);

static int
lend_atom(struct e4_atom_type **atom, unsigned *lend, int *none, int call,
	  struct parser_type *parser_data)
{
    int status = 0;
    const struct e4_odbx_type *const *list;

    list = parser_data->attributes_data->odb1_data;
    if (!list) {
	status = _x1f4_e4_080704_error_g(parser_data->back_data);
    } else {
	struct e4_atom_type *atom_data;
	struct e4_odb1_type *odb1_data, *odb1_miss = NULL;

	atom_data = *atom;

	odb1_data = atom_data->odb1;
	while (odb1_data) {
	    const char *e;
	    struct text_type *text_data;
	    unsigned seek;

	    text_data = (struct text_type *) (odb1_data + 1);

	    e = text_data->data;
	    seek = text_data->size;
	    while (seek) {
		int mark;
		const struct e4_odbx_type *const *odbx, *odbx_data,
		    *odbx_text = NULL;
		unsigned toxic = 0;

		odbx = list;

		odbx_data = *odbx;
		while (odbx_data) {
		    unsigned shift;

		    shift = odbx_data->size;
		    if (toxic < shift) {
			if (seek < shift) {
			} else {
			    const char *name;

			    name = odbx_data->name;
			    if (memcmp((void *) (e - shift), name, shift)) {
			    } else {
				odbx_text = odbx_data;
				toxic = shift;
			    }
			}
		    }

		    odbx++;
		    odbx_data = *odbx;
		}

		if (odbx_text) {
		} else {
		    status = _x1f4_e4_080704_error_i(parser_data->back_data);
		    break;
		}

		mark = odbx_text->priority;
		if (call < mark) {
		    struct e4_atom_type *atom_text;
		    struct e4_hack_type *hack_data;
		    struct e4_odb1_type *odb1_half = NULL;
		    unsigned class, half, part;

		    atom_text = atom_data + 1;
		    part = *lend - 1;
		    for (; part; part--) {
			odbx_data = atom_text->odb2._1st;
			if (odbx_data->priority < mark) {
			    atom_text++;
			} else {
			    break;
			}
		    }

		    half = text_data->size - seek;

		    if (half) {
			status = M_LINK
			    (parser_data, &odb1_half,
			     sizeof(struct e4_odb1_type)
			     + sizeof(struct text_type));
			if (status) {
			    status = ALLOC_ERROR;

			    seek = ~0;

			    break;
			} else {
			    struct text_type *text_half;

			    text_half = (struct text_type *) (odb1_half + 1);

			    odb1_half->odb1_data = NULL;

			    text_half->data = text_data->data;
			    text_half->size = half;

			    text_data->data = e;
			    text_data->size = seek;
			}
		    }

		    atom_text++;

		    class = atom_text - atom_data;

		    status = M_LINK
			(parser_data, &hack_data, sizeof(struct e4_hack_type));
		    if (status) {
			status = ALLOC_ERROR;

			if (odb1_half) {
			    M_FREE(parser_data, odb1_half);
			}
		    } else {
			struct e4_atom_type *atom_news;
			unsigned link;

			link = class * sizeof(struct e4_atom_type);

			status = M_LINK(parser_data, &atom_news, link);
			if (status) {
			    status = ALLOC_ERROR;

			    M_FREE(parser_data, hack_data);
			    if (odb1_half) {
				M_FREE(parser_data, odb1_half);
			    }
			} else {
			    memcpy(atom_news, atom_data, link);

			    hack_data->atoms = atom_news;
			    hack_data->bits = READ_ONLY;
			    hack_data->count = class;

			    atom_data->data.hack.data = hack_data;
			    atom_data->type = THIS;

			    if (odb1_miss) {
				atom_data->odb1 = odb1_data;
				if (odb1_half) {
				    odb1_miss->odb1_data = odb1_half;
				} else {
				    odb1_miss->odb1_data = NULL;
				}
			    } else {
				if (odb1_half) {
				    atom_news->odb1 = odb1_half;
				} else {
				    atom_news->odb1 = NULL;
				}
			    }

			    class--;

			    atom_news += class;

			    atom_data->odb2._1st = atom_news->odb2._1st;

			    atom_data++;

			    part = *lend - class;

			    memmove(atom_data, atom_text,
				    part * sizeof(struct e4_atom_type));

			    *none = 0;
			    *lend = part;
			}
		    }

		    seek = ~0;

		    break;
		}

		e -= toxic;
		seek -= toxic;
	    }

	    if (seek | status) {
		break;
	    }

	    odb1_miss = odb1_data;

	    odb1_data = odb1_data->odb1_data;
	}
    }

    return status;
}


int
_x1f4_e4_trip_hack(struct e4_hack_type *hack_data, void *parser)
{
    int status = 0;
    struct e4_atom_type *atom_data;
    unsigned count;

    count = hack_data->count;

    atom_data = hack_data->atoms;
    count--;
    while (count) {
	if (atom_data->odb1) {
	    int none = 1;
	    const struct e4_odbx_type *odbx_data;

	    odbx_data = atom_data->odb2._1st;

	    status = lend_atom
		(&atom_data, &count, &none, odbx_data->priority, parser);
	    if (status) {
		break;
	    } else {
		if (none) {
		    count--;
		    atom_data++;
		}
	    }
	} else {
	    count--;
	    atom_data++;
	}
    }

    count += atom_data - (struct e4_atom_type *) hack_data->atoms + 1;
    hack_data->count = count;

    if (status) {
    } else {
	struct parser_type *parser_data;

	parser_data = parser;
	if (parser_data->bits & PACK_ESTORAGE) {
	    M_MODE(parser_data, &hack_data->atoms,
		   count * sizeof(struct e4_atom_type));
	}
    }

    return status;
}
