/*
 * file-types.h
 * Copyright (C) 2008, 2009, 2013, 2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_file_types_H__
#define __LIBx1f4_file_types_H__

#include <sys/types.h>

#include <trans.h>

typedef struct adfile_type {
    unsigned slip;
    void *data, *text;
} adfile_type;

typedef struct ajdata_type {
    struct ajdata_type *last, *next;
} ajdata_type;

typedef struct ajfile_type {
    int fd;
    struct ajdata_type *last, *next;
    unsigned lead, slip;
    void *trans;
} ajfile_type;

typedef struct chance_type {
    struct chance_type *chance_data;
    unsigned size;
} chance_type;

typedef struct edfile_type {
    unsigned slip;
    void *data, *text;
} edfile_type;

typedef union sdfile_type {
    int fa;
    struct adfile_type ra;
    struct ajfile_type rb;
    struct edfile_type rc;
} sdfile_type;

typedef struct sffile_type {
    int (*fine)(void *), (*kate)(void *, off_t), (*lose)(void *),
	(*pull)(void *, void *, unsigned), (*push)(void *, void *, unsigned);
    off_t (*seek)(void *, off_t, int);
} sffile_type;

typedef struct aofile_type {
    const struct sffile_type *sf;
    union sdfile_type sd;
    unsigned bits, ever, size;
} aofile_type;

typedef struct eifile_type {
    const struct sffile_type *sf;
    union sdfile_type sd;
    unsigned ever, size;
} eifile_type;

#endif				/* __LIBx1f4_file_types_H__ */
