/*
 * lxcast-i.0.c
 * Copyright (C) 2008-2015, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>
#include <string.h>

#include <e4.h>
#include <lxcall.h>
#include <lxcast-fold.h>
#include <lxcast-inter.h>
#include <lxcast-types.h>

#define LINE_C0_BITS \
    X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE | X1f4_E4_TEXT_LINK		      \
    | X1f4_E4_XSET_LINK
#define LINE_C1_BITS \
    X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE | X1f4_E4_SLIP_LIST		      \
    | X1f4_E4_TEXT_LINK | X1f4_E4_XSET_LINK
#define LINE_C__BITS \
    X1f4_E4_KEEP_CALL | X1f4_E4_POST_TYPE | X1f4_E4_TEXT_LINK

#define SIDE_CO_BITS \
    X1f4_E4_BACK_LINK | X1f4_E4_CALL_XSET | X1f4_E4_E2ND_LINK
#define SIDE_CL_BITS \
    X1f4_E4_BACK_LINK | X1f4_E4_LEFT_XSET | X1f4_E4_E2ND_LINK

#define I_BILL(b)			(*((X1f4_E4_C_BILL *) (b)))
#define I_MODE(i)			(*((X1f4_E4_C_MODE *) (i)))
#define I_REAL(r)			(*((X1f4_E4_C_REAL *) (r)))
#define I_TEXT(t)			(*((X1f4_E4_C_TEXT *) (t)))
#define I_USER(u)			(*((X1f4_E4_C_USER *) (u)))

#define a7axx_x_c			_libx1f4i0_lxcast_y_indexed
#define c7axx_f_c			_libx1f4i0_lxcast_q_indexed
#define c7axx_f_f			_libx1f4i0_lxcast_r_indexed

#define l_BILL(e, output) \
    {									      \
	X1f4_E4_C_BILL *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define l_MODE(e, output) \
    {									      \
	X1f4_E4_C_MODE *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define l_REAL(e, output) \
    {									      \
	X1f4_E4_C_REAL *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define l_TEXT(e, output) \
    {									      \
	X1f4_E4_C_TEXT *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

#define MAKE_DOUBLE(a, b)		a b
#define MAKE_SINGLE(a, b)		a

#define MAKE_VIOLET(a, b)		MAKE_DOUBLE(MAKE_DOUBLE(a, b), "=")

static int a2cxx_b_c(void *, void *, void **);
static int a2cxx_m_c(void *, void *, void **);
static int a2cxx_r_c(void *, void *, void **);
static int a2exx_b_c(void *, void *, void **);
static int a2exx_m_c(void *, void *, void **);
static int a2exx_r_c(void *, void *, void **);
static int a5fxx_b_c(void *, void *, void **);
static int a5fxx_m_c(void *, void *, void **);
static int a66xx_b_c(void *, void *, void **);
static int a66xx_m_c(void *, void *, void **);
static int a68xx_b_c(void *, void *, void **);
static int a68xx_m_c(void *, void *, void **);
static int a69xx_b_c(void *, void *, void **);
static int a69xx_m_c(void *, void *, void **);
static int a6axx_b_c(void *, void *, void **);
static int a6axx_m_c(void *, void *, void **);
static int a70xx_b_c(void *, void *, void **);
static int a70xx_m_c(void *, void *, void **);
static int a70xx_r_c(void *, void *, void **);
static int a71xx_b_c(void *, void *, void **);
static int a71xx_m_c(void *, void *, void **);
static int a71xx_r_c(void *, void *, void **);
static int a75xx_b_c(void *, void *, void **);
static int a75xx_m_c(void *, void *, void **);
static int a7axx_b_c(void *, void *, void **);
static int a7axx_m_c(void *, void *, void **);
static int a7axx_r_c(void *, void *, void **);
static int a7axx_t_c(void *, void *, void **);

static const int c_____b_c[] = {
/* *INDENT-OFF* */
    X1f4_E4_BILL,
    X1f4_E4_CASE,
    X1f4_E4_POST_XSET,
    0
/* *INDENT-ON* */
}, c_____f_c[] = {
/* *INDENT-OFF* */
    X1f4_E4_SLIP,
    X1f4_E4_CASE,
    X1f4_E4_POST_XSET,
    0
/* *INDENT-ON* */
}, c_____f_f[] = {
/* *INDENT-OFF* */
    X1f4_E4_CALL,
    X1f4_E4_CALL,
    X1f4_E4_POST_XSET,
    0
/* *INDENT-ON* */
}, c_____m_c[] = {
/* *INDENT-OFF* */
    X1f4_E4_MODE,
    X1f4_E4_CASE,
    X1f4_E4_POST_XSET,
    0
/* *INDENT-ON* */
}, c_____r_c[] = {
/* *INDENT-OFF* */
    X1f4_E4_REAL,
    X1f4_E4_CASE,
    X1f4_E4_POST_XSET,
    0
/* *INDENT-ON* */
}, c_____t_c[] = {
/* *INDENT-OFF* */
    X1f4_E4_TEXT,
    X1f4_E4_CASE,
    X1f4_E4_POST_XSET,
    0
/* *INDENT-ON* */
};
static const struct x1f4_2ndscope_type side_2[] = {
/* *INDENT-OFF* */
    {	X1f4_E4_CALL_LINK,	NULL,
	NULL,			NULL				}
/* *INDENT-ON* */
};
const struct x1f4_function_type _libx1f4i0_lxcast_line_0[] = {
/* *INDENT-OFF* */
    {	"copy",			c7axx_f_f,
	X1f4_E4_CALL,		c_____f_f,	2,
	LINE_C0_BITS,				4		},
    {	"copy",			c7axx_f_c,
	X1f4_E4_CALL,		c_____f_c,	2,
	LINE_C1_BITS,				4		},
    {	"copy",			a7axx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C0_BITS,				4		},
    {	"copy",			a7axx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C0_BITS,				4		},
    {	"copy",			a7axx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_C0_BITS,				4		},
    {	"copy",			a7axx_t_c,
	X1f4_E4_TEXT,		c_____t_c,	2,
	LINE_C0_BITS,				4		},
    {	"fold",			a2cxx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a2cxx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a2cxx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a2exx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a2exx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a2exx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a5fxx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a5fxx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a66xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a66xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a68xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a68xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a69xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a69xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a6axx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a6axx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a70xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a70xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a70xx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a71xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a71xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a71xx_r_c,
	X1f4_E4_REAL,		c_____r_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a75xx_b_c,
	X1f4_E4_BILL,		c_____b_c,	2,
	LINE_C__BITS,				4		},
    {	"fold",			a75xx_m_c,
	X1f4_E4_MODE,		c_____m_c,	2,
	LINE_C__BITS,				4		}
/* *INDENT-ON* */
}, _libx1f4i0_lxcast_line_1[] = {
/* *INDENT-OFF* */
    {	"copy",			a7axx_x_c,
	0,			NULL,		2,
	LINE_C0_BITS,				4		}
/* *INDENT-ON* */
};
const struct x1f4_operator_type _libx1f4i0_lxcast_side_o[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	X1f4_E4_CALL,		c_____f_f,
	SIDE_CO_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	X1f4_E4_CALL,		c_____f_c,
	SIDE_CO_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CL_BITS,		1,
	NULL,			NULL				},
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	X1f4_E4_TEXT,		c_____t_c,
	SIDE_CL_BITS,		1,
	NULL,			NULL				},
    {	MAKE_DOUBLE("/", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("/", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("/", "="),	NULL,		0400,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("*", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("*", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("*", "="),	NULL,		0400,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("%", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("%", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_VIOLET("<", "<"),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		3,
	NULL,			NULL				},
    {	MAKE_VIOLET("<", "<"),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		3,
	NULL,			NULL				},
    {	MAKE_DOUBLE("&", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("&", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("^", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("^", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("|", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("|", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("+", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("+", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("+", "="),	NULL,		0400,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("-", "="),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("-", "="),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_DOUBLE("-", "="),	NULL,		0400,
	X1f4_E4_REAL,		c_____r_c,
	SIDE_CL_BITS,		2,
	NULL,			NULL				},
    {	MAKE_VIOLET(">", ">"),	NULL,		0400,
	X1f4_E4_BILL,		c_____b_c,
	SIDE_CL_BITS,		3,
	NULL,			NULL				},
    {	MAKE_VIOLET(">", ">"),	NULL,		0400,
	X1f4_E4_MODE,		c_____m_c,
	SIDE_CL_BITS,		3,
	NULL,			NULL				}
/* *INDENT-ON* */
}, _libx1f4i0_lxcast_side_x[] = {
/* *INDENT-OFF* */
    {	MAKE_SINGLE("=", " "),	NULL,		0400,
	0,			NULL,
	SIDE_CL_BITS,		1,
	NULL,			NULL				}
/* *INDENT-ON* */
};
const unsigned _libx1f4i0_lxcast_size_o =
    sizeof(_libx1f4i0_lxcast_side_o) / sizeof(struct x1f4_operator_type);

static int
a2cxx_b_c(void *context, void *output, void **input)
{
    EVER_FOLD(/=, BILL);
}


static int
a2cxx_m_c(void *context, void *output, void **input)
{
    EVER_FOLD(/=, MODE);
}


static int
a2cxx_r_c(void *context, void *output, void **input)
{
    EVER_FOLD(/=, REAL);
}


static int
a2exx_b_c(void *context, void *output, void **input)
{
    EVER_FOLD(*=, BILL);
}


static int
a2exx_m_c(void *context, void *output, void **input)
{
    EVER_FOLD(*=, MODE);
}


static int
a2exx_r_c(void *context, void *output, void **input)
{
    EVER_FOLD(*=, REAL);
}


static int
a5fxx_b_c(void *context, void *output, void **input)
{
    EVER_LONG(%=, BILL);
}


static int
a5fxx_m_c(void *context, void *output, void **input)
{
    EVER_LONG(%=, MODE);
}


static int
a66xx_b_c(void *context, void *output, void **input)
{
    EVER_LONG(<<=, BILL);
}


static int
a66xx_m_c(void *context, void *output, void **input)
{
    EVER_LONG(<<=, MODE);
}


static int
a68xx_b_c(void *context, void *output, void **input)
{
    EVER_LONG(&=, BILL);
}


static int
a68xx_m_c(void *context, void *output, void **input)
{
    EVER_LONG(&=, MODE);
}


static int
a69xx_b_c(void *context, void *output, void **input)
{
    EVER_LONG(^=, BILL);
}


static int
a69xx_m_c(void *context, void *output, void **input)
{
    EVER_LONG(^=, MODE);
}


static int
a6axx_b_c(void *context, void *output, void **input)
{
    EVER_LONG(|=, BILL);
}


static int
a6axx_m_c(void *context, void *output, void **input)
{
    EVER_LONG(|=, MODE);
}


static int
a70xx_b_c(void *context, void *output, void **input)
{
    EVER_FOLD(+=, BILL);
}


static int
a70xx_m_c(void *context, void *output, void **input)
{
    EVER_FOLD(+=, MODE);
}


static int
a70xx_r_c(void *context, void *output, void **input)
{
    EVER_FOLD(+=, REAL);
}


static int
a71xx_b_c(void *context, void *output, void **input)
{
    EVER_FOLD(-=, BILL);
}


static int
a71xx_m_c(void *context, void *output, void **input)
{
    EVER_FOLD(-=, MODE);
}


static int
a71xx_r_c(void *context, void *output, void **input)
{
    EVER_FOLD(-=, REAL);
}


static int
a75xx_b_c(void *context, void *output, void **input)
{
    EVER_LONG(>>=, BILL);
}


static int
a75xx_m_c(void *context, void *output, void **input)
{
    EVER_LONG(>>=, MODE);
}


static int
a7axx_b_c(void *context, void *output, void **input)
{
    EVER_COPY(BILL);
}


static int
a7axx_m_c(void *context, void *output, void **input)
{
    EVER_COPY(MODE);
}


static int
a7axx_r_c(void *context, void *output, void **input)
{
    EVER_COPY(REAL);
}


static int
a7axx_t_c(void *context, void *output, void **input)
{
    int status;
    struct dxcast_type *object_data;
    struct lxcast_type *lxcast_data;

    lxcast_data = context;

    object_data = I_USER(input[1]);

    if (object_data->type ^ X1f4_E4_TEXT) {
	status = _libx1f4i0_lxcast_stat_luck
	    (lxcast_data, 1, object_data->type, X1f4_E4_TEXT);
    } else {
	X1f4_E4_C_TEXT text;

	status = 0;

	text = I_TEXT(&object_data->data);

	status = _libx1f4i0_lxcall_swap_text
	    (&lxcast_data->link_v, &lxcast_data->link_m,
	     &lxcast_data->link_e, *input, text, strlen(text) + 1);
	if (status) {
	} else {
	    l_TEXT(I_TEXT(input[0]), output);
	}
    }

    return status;
}
