/*
 * lxdash-a.e.c
 * Copyright (C) 2008-2015, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <d_fset.h>
#include <e4.h>
#include <er.h>
#include <exerrors.h>
#include <lxcall.h>
#include <lxdash-inter.h>
#include <lxdash-types.h>

#define lxdash(lxdash) \
    ((struct lxdash_type *) (lxdash))

int
_libx1f4i0_lxdash_stat_ends(void *lxdash, int tail, unsigned found,
			    unsigned expected)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("mismatched type (", 17);
    PUSH_TRAP(found, lxdash(lxdash));
    PUSH_DATA(" found, ", 8);
    PUSH_TRAP(expected, lxdash(lxdash));
    PUSH_DATA(" expected) for ", 15);
    if (tail) {
	PUSH_DATA("last", 4);
    } else {
	PUSH_DATA("first", 5);
    }
    PUSH_DATA(" key", 4);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxdash_stat_even(void *lxdash)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("even number of parameters", 25);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxdash_stat_ever(void *lxdash)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("key exists", 10);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxdash_stat_flat(void *lxdash, long text)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("numeric type expected for the `", 31);
    PUSH_LONG(text);
    PUSH_DATA("' key", 5);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxdash_stat_free(void *lxdash, int shadow)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("cannot free memory", 18);

    POST_EEER();

    return _libx1f4i0_lxcall_land_slip(X1f4_EX_CRITICAL, shadow);
}


int
_libx1f4i0_lxdash_stat_lead(void *lxdash, unsigned type)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("will not box non referable, non intrinsic object of type ", 57);
    PUSH_TRAP(type, lxdash(lxdash));

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxdash_stat_link(void *lxdash)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("cannot allocate memory", 22);

    POST_EEER();

    return X1f4_EX_CRITICAL;
}


int
_libx1f4i0_lxdash_stat_line(void *lxdash, int shadow, int error)
{
    int status;

    if (error == X1f4_BQFSET_DECK_ERROR) {
	status = _libx1f4i0_lxdash_stat_null(lxdash);
    } else {
	if (error == X1f4_BQFSET_EVER_MATCH) {
	    status = _libx1f4i0_lxdash_stat_ever(lxdash);
	} else {
	    if (error == X1f4_BQFSET_FREE_ERROR) {
		status = _libx1f4i0_lxdash_stat_free(lxdash, 0);
	    } else {
		if (error == X1f4_BQFSET_LINK_ERROR) {
		    status = _libx1f4i0_lxdash_stat_link(lxdash);
		} else {
		    if (error == X1f4_BQFSET_MODE_ERROR) {
			status = _libx1f4i0_lxdash_stat_mode(lxdash);
		    } else {
			/*
			 * there is defined a _TEXT_ERROR condition, but the
			 * condition is never met when writing this.
			 */
			status = X1f4_EX_CRITICAL;
		    }
		}
	    }
	}
    }

    return _libx1f4i0_lxcall_land_slip(status, shadow);
}


int
_libx1f4i0_lxdash_stat_lock(void *lxdash, int mode)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("non intrinsic integer ", 22);
    PUSH_MODE(mode + 1);
    PUSH_RANK(mode);
    PUSH_DATA(" index argument", 15);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxdash_stat_miss(void *lxdash, long text)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("no mapping for key `", 20);
    PUSH_LONG(text);
    PUSH_DATA("'", 1);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxdash_stat_mode(void *lxdash)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("cannot reallocate memory", 24);

    POST_EEER();

    return X1f4_EX_CRITICAL;
}


int
_libx1f4i0_lxdash_stat_null(void *lxdash)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("void index", 10);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxdash_stat_odds(void *lxdash)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("odd number of parameters", 24);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxdash_stat_post(void *lxdash, long text)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("key `", 5);
    PUSH_LONG(text);
    PUSH_DATA("' exists", 8);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}


int
_libx1f4i0_lxdash_stat_text(void *lxdash, int error, long text)
{
    int status;

    if (error == X1f4_BQFSET_EVER_MATCH) {
	status = _libx1f4i0_lxdash_stat_post(lxdash, text);
    } else {
	if (1) {
	    status = _libx1f4i0_lxdash_stat_line(lxdash, 0, error);
	}
    }

    return status;
}


int
_libx1f4i0_lxdash_stat_type(void *lxdash, long text, unsigned found,
			    unsigned expected)
{
    LINE_EEER(lxdash(lxdash)->link_e);

    PUSH_DATA("mismatched type (", 17);
    PUSH_TRAP(found, lxdash(lxdash));
    PUSH_DATA(" found, ", 8);
    PUSH_TRAP(expected, lxdash(lxdash));
    PUSH_DATA(" expected) for `", 16);
    PUSH_LONG(text);
    PUSH_DATA("' key", 5);

    POST_EEER();

    return X1f4_EX_CAN_CONTINUE;
}
