/*
 * lxdash-a.s.c
 * Copyright (C) 2008-2015, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>

#include <string.h>

#include <d_fset.h>
#include <lxdash-inter.h>
#include <lxdash-types.h>

#if SIZEOF_VOID_P == 8
# define HALF				040
#else
# define HALF				020
#endif				/* SIZEOF_VOID_P == 8 */

#define SECOND(___v)			(((struct effect_type *) (___v))->mode)

static int fine_fare(void *, void *);
static int fine_look(void *, void *, unsigned *);
static int fine_move(void *, void *, unsigned);
static int fine_pick(void *, void *, unsigned *);

static int
fine_fare(void *text, void *node)
{
    X1f4_E4_C_MODE u, v;

    u = SECOND(text);
    v = SECOND(node);

    return u < v ? -1 : u ^ v ? 1 : 0;
}


static int
fine_look(void *text, void *node, unsigned *pick)
{
    int status;
    unsigned size;

    size = *(integral_q *) node;
    if (size) {
	X1f4_E4_C_MODE u, v;
	struct effect_type *effect_call;

	u = SECOND(text);

	effect_call = node;
	effect_call++;

	while (size ^ 1) {
	    struct effect_type *effect_slip;
	    unsigned half;

	    half = size >> 1;
	    effect_slip = effect_call + half;

	    v = SECOND(effect_slip);
	    if (u < v) {
		size = half;
	    } else {
		effect_call = effect_slip;
		size -= half;
	    }
	}

	v = SECOND(effect_call);
	if (v ^ u) {
	    status = 0;

	    if (u < v) {
		effect_call--;
	    } else {
	    }
	} else {
	    status = 1;
	}

	*pick = effect_call - (struct effect_type *) node;
    } else {
	*pick = 0;

	status = 0;
    }

    return status;
}


static int
fine_move(void *target, void *source, unsigned count)
{
    memmove(target, source, count * sizeof(struct effect_type));

    return 0;
}


static int
fine_pick(void *text, void *node, unsigned *pick)
{
    X1f4_E4_C_MODE u;
    int status;
    struct effect_type *effect_call;
    unsigned half = HALF, slip = 0;

    u = SECOND(text);

    effect_call = node;

    do {
	if (u < SECOND(effect_call + slip + half)) {
	} else {
	    slip += half;
	}

	half >>= 1;
    } while (half ^ 1);

    effect_call += slip;

    if (*(integral_q *) node & (integral_q) 2 << slip) {
	if (u < SECOND(effect_call + 1)) {
	    if (slip) {
		if (u ^ SECOND(effect_call)) {
		    status = 0;
		} else {
		    status = 1;
		}
	    } else {
		status = 0;
	    }
	} else {
	    slip++;
	    if (u ^ SECOND(effect_call + 1)) {
		status = 0;
	    } else {
		status = 1;
	    }
	}

	*pick = slip;
    } else {
	*pick = slip + 1;
	if (slip) {
	    if (u ^ SECOND(effect_call)) {
		status = 0;
	    } else {
		status = 1;
	    }
	} else {
	    if (1) {
		status = 0;
	    }
	}
    }

    return status;
}


int
_libx1f4i0_lxdash_fast_line(void *lxdash, void *dxdash)
{
    int status;
    struct lxdash_type *lxdash_data;
    struct _libx1f4i0_bqfset_type bqfset;

    lxdash_data = lxdash;

    bqfset.fare = fine_fare;
    bqfset.look = fine_look;
    bqfset.move = fine_move;
    bqfset.pick = fine_pick;

    bqfset.trans = &lxdash_data->link_m;

    bqfset.size = sizeof(struct effect_type);

    ((struct dxdash_type *) dxdash)->dash = post_dxdash(dxdash);

    status = _libx1f4i0_fast_lxfset
	(((struct dxdash_type *) dxdash)->dash,
	 X1f4_BQFSET_SIZE_FRAME | X1f4_BQFSET_TRANS_MASK, &bqfset);
    if (status) {
	status = _libx1f4i0_lxdash_stat_line(lxdash_data, 0, status);
    }

    return status;
}
