/*
 * lxdash-types.h
 * Copyright (C) 2008-2015, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_lxdash_types_H__
#define __LIBx1f4_lxdash_types_H__

#include <config.h>

#include <dt.h>
#include <e4.h>
#include <fd.h>
#include <frame.h>
#include <ln.h>
#include <lxcast.h>
#include <nl.h>
#include <nt.h>
#include <track.h>
#include <trans.h>

#if SIZEOF_VOID_P == SIZEOF_LONG
# define integral_q			unsigned long
#else
# define integral_q			unsigned
#endif				/* SIZEOF_VOID_P == SIZEOF_LONG */

#define excase_type			x1f4_nodelink_type
#define exline_type			x1f4_nodeline_type

#define cxline_type			x1f4_leadnode_type

#define lxtype_type			x1f4_nodetype_type

#define here_NULL			((void *) 0)

#define here_offsetof(type, ceil) \
    ((integral_q) (&((type *) here_NULL)->ceil))

#define lock_excase(dxdash_text) \
    ((struct excase_type *) &lock_cxline((dxdash_text))->link)

#define lock_exline(dxdash_text) \
    ((struct exline_type *) &lock_cxline((dxdash_text))->line)

#define lock_cxline(dxdash_text) \
    ((struct cxline_type *)						      \
     ((char *) (dxdash_text) - sizeof(struct cxline_type)))

#define near_dxdash(excase_text) \
    ((struct dxdash_type *)						      \
     ((char *) near_cxline((excase_text)) + sizeof(struct cxline_type)))

#define near_exline(excase_text) \
    ((struct exline_type *) &near_cxline((excase_text))->line)

#define near_cxline(excase_text) \
    ((struct cxline_type *)						      \
     ((char *) (excase_text) - here_offsetof(struct cxline_type, link)))

#define pick_dxdash(fxline_text) \
    ((struct dxdash_type *)						      \
     ((char *) (fxline_text) + sizeof(struct cxline_type)))

#define post_dxdash(dxdash_text) \
    ((struct dxdash_type *) (dxdash_text) + 1)

#define __struct_dxdash_type_alloc_size__ \
    sizeof(struct cxline_type) + sizeof(struct dxdash_type)

typedef struct dxdash_type {
    unsigned size;
    void *dash;
} dxdash_type;

typedef struct effect_type {
    X1f4_E4_C_MODE mode;
    int link, type;
    struct fxdata_type fxdata;
} effect_type;

/*
 * NOTES
 *
 *.there may be a need for a temporary memory allocator in addition to the
 * permanent memory allocator (the `link_m' one).
 *.there may be a need for a temporary objects destroyer in addition  to the
 * temporary memory allocator.
 *.some error recorder in addition to the error dumper is in good order.  such
 * recorder would record the error type (this library identification -
 * application side, since such identification is created by the application)
 * and the error details (library error type and details)
 */
typedef struct lxdash_type {
    struct x1f4_track_type link_e;
    struct {
	const struct x1f4_eelookup_type *eelookup_data;
    } trap_e;
    struct {
	void *data, *side, *text;
    } link_f;
    struct {
	int data;
    } link_i;
    struct {
	void *data;
    } link_l;
    struct trans_type link_m;
    struct {
	void *node;
    } link_n;
    struct {
	int (*call) (void *, struct x1f4_nodelink_type *),
	    (*fine) (void *, struct x1f4_nodelink_type **,
		     struct x1f4_nodelink_type *),
	    (*miss) (void *, struct x1f4_nodelink_type **,
		     struct x1f4_nodelink_type *);
	void *text;
    } link_r;
    struct {
	void *fine;
    } link_o;
    struct {
	struct x1f4_datatype_type datatype[2];
    } link_t;
    struct x1f4_frame_type link_v;
    struct trans_type link_w;
} lxdash_type;

typedef struct lxmiss_type {
    union {
	unsigned miss;
	void *data;
    } data;
} lxmiss_type;

typedef struct lxtext_type {
    struct lxtype_type lxtype;
    void *text;
} lxtext_type;

typedef struct screen_type {
    struct x1f4_track_type link_e;
    struct {
	const struct x1f4_eelookup_type *eelookup_data;
    } trap_e;
    struct {
	int type;
    } link_l;
    struct {
	int (*free) (void *, void *), (*link) (void *, void **, unsigned),
	    (*mode) (void *, void **, unsigned);
	void *data;
    } link_m;
    struct {
	int (*call) (void *, struct x1f4_nodelink_type *),
	    (*fine) (void *, struct x1f4_nodelink_type **,
		     struct x1f4_nodelink_type *),
	    (*miss) (void *, struct x1f4_nodelink_type **,
		     struct x1f4_nodelink_type *);
	void *text;
    } link_r;
    struct {
	void *fine;
    } link_o;
    struct {
	unsigned miss;
	const void *data;
    } link_t;
    struct x1f4_frame_type link_v;
    struct trans_type link_w;
    struct {
	struct x1f4_linetrap_type *linetrap_data;
    } link_y;
} screen_type;

#endif				/* __LIBx1f4_lxdash_types_H__ */
