/*
 * lxtrap-i.w.c
 * Copyright (C) 2011-2015, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stddef.h>

#include <e4.h>
#include <exerrors.h>
#include <fd.h>
#include <lxcall.h>
#include <lxcast.h>
#include <lxtrap-inter.h>
#include <lxtrap-types.h>
#include <retain.h>

#if __RETAIN_OBJECT_DATA__
# error data for created objects is not retained, as probably should have been
#endif				/* __RETAIN_OBJECT_DATA__ */

#define I_USER(t)			(*((X1f4_E4_C_USER *) (t)))

#define l_MODE(e, output) \
    {									      \
	X1f4_E4_C_MODE *l;						      \
									      \
	l = (void *) (output);						      \
	*l = (e);							      \
    }

extern const void *const x1f4_a1_walk_link;

static int pick_call(void *, int, void *, void **, void **, void *,
		     struct lxtrap_type *, struct x1f4_linetext_type *, int);
static int slip_call(void *, int, void *, void **, void **, void *,
		     struct lxtrap_type *, struct x1f4_linetext_type *, int);

static int
pick_call(void *output, int land, void *report, void **input,
	  void **track, void *side, struct lxtrap_type *lxtrap_data,
	  struct x1f4_linetext_type *linetext_data, int frame)
{
    int status;
    struct fxdata_type missed;

    status = _libx1f4i0_lxtrap_pick_call
	(&missed, 2, input, track, side, lxtrap_data, linetext_data, frame);
    if (status) {
	if (status == X1f4_EX_CAN_CONTINUE) {
	    status = 0;
	    l_MODE(-1, output);
	} else {
	}
    } else {
	status = _libx1f4i0_lxtrap_lock_port
	    (land, report, &missed, linetext_data->function.type, lxtrap_data);
	if (status) {
	} else {
	    l_MODE(0, output);
	}
    }

    return status;
}


static int
slip_call(void *output, int land, void *report, void **input,
	  void **track, void *side, struct lxtrap_type *lxtrap_data,
	  struct x1f4_linetext_type *linetext_data, int frame)
{
    return _libx1f4i0_lxtrap_slip_call
	(output, land, report, 2, input, track, side, lxtrap_data,
	 linetext_data, frame);
}


int
_libx1f4i0_lxtrap_wrap_call(void *context, void *output, void **input,
			    int frame)
{
    int linked, shift, status;
    struct lxtrap_type *lxtrap_data;
    struct x1f4_dxcast_type *dxcast_data;
    void **track;

    linked = output == x1f4_a1_walk_link;

    lxtrap_data = context;

    if (linked) {
	track = input[1];
    } else {
	track = input;
    }

    shift = frame ^ 2 ? 0 : 1;

    dxcast_data = I_USER(track[shift + 1]);

    if (X1f4_E4_CALL < dxcast_data->type) {
	int type, wind;
	struct x1f4_function_type *function_data;
	struct x1f4_linetext_type *linetext_data;
	void *meta;

	linetext_data = I_USER(&dxcast_data->data);

	wind = linetext_data->function.type;

	meta = *(track - 1);

	function_data = ((void **) meta)[X1f4_E4_RENDITION];

	type = function_data->args[shift];
	if (wind == type || wind == X1f4_E4_CASE) {
	    if (type ^ X1f4_E4_CASE) {
		void *missed;

		missed = track[shift];

		shift += 2;

		track += shift;

		status = _libx1f4i0_lxtrap_even_list
		    (&linetext_data->function, function_data, shift, &track,
		     lxtrap_data);
		if (status) {
		} else {
		    if (linetext_data->function.function) {
			int (*call) (void *, int, void *, void **, void **,
				     void *, struct lxtrap_type *,
				     struct x1f4_linetext_type *, int);

			if (linked) {
			    call = slip_call;
			} else {
			    call = pick_call;
			}

			track[-1] = meta;

			status = call
			    (output, type, missed, input, track, function_data,
			     lxtrap_data, linetext_data, frame);
		    } else {
			status = _libx1f4i0_lxtrap_stat_null
			    (lxtrap_data, "wrap", 4);
		    }
		}
	    } else {
		status = _libx1f4i0_lxtrap_stat_cast(lxtrap_data);
	    }
	} else {
	    status = _libx1f4i0_lxtrap_stat_post
		(lxtrap_data, type, linetext_data->function.type);
	}
    } else {
	status = _libx1f4i0_lxtrap_stat_call(lxtrap_data, 0, "wrap", 4);
    }

    return status;
}
