/*
 * qsrate.h
 * Copyright (C) 2010-2014, Ciprian Niculescu
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __LIBx1f4_QSRATE_H__
#define __LIBx1f4_QSRATE_H__

#include <errors.h>
#include <qscc-names.h>

/*
 * attribute bit definitions
 */
#define X1f4_QSRATE_FOUR_FRAME		(1u << 0x00)
#define X1f4_QSRATE_RATE_FRAME		(1u << 0x01)
#define X1f4_QSRATE_TRANS_MASK		(1u << 0x02)

/*
 * error definitions
 */
#define X1f4_QSRATE_CALL_ERROR		LIBx1f4L2_CALL_ERROR
#define X1f4_QSRATE_DECK_ERROR		LIBx1f4L2_DECK_ERROR
#define X1f4_QSRATE_FREE_ERROR		LIBx1f4L2_FREE_ERROR
#define X1f4_QSRATE_LINK_ERROR		LIBx1f4L2_LINK_ERROR
#define X1f4_QSRATE_MODE_ERROR		LIBx1f4L2_MODE_ERROR
#define X1f4_QSRATE_SIZE_ERROR		LIBx1f4L2_SIZE_ERROR
#define X1f4_QSRATE_TEXT_ERROR		LIBx1f4L2_TEXT_ERROR

typedef struct x1f4_qsrate_type {
    unsigned quarter, rate, size;
    void *trans;
} x1f4_qsrate_type;

#if !defined __x1f4_trans_type_defined__
typedef struct x1f4_trans_type {
    int (*free) (void *, void *), (*link) (void *, void **, unsigned),
	(*mode) (void *, void **, unsigned);
    void *data;
} x1f4_trans_type;
#endif				/* !__x1f4_trans_type_defined__ */

#if !defined __x1f4_trans_type_defined__
# define __x1f4_trans_type_defined__
#endif				/* !__x1f4_trans_type_defined__ */

extern int x1f4_call_qsrate(unsigned *);
extern int x1f4_case_qsrate(void *);
extern int x1f4_dash_qsrate(void *, unsigned, unsigned);
extern int x1f4_ever_qsrate(void *, void **);
extern int x1f4_fast_qsrate(void *, unsigned, struct x1f4_qsrate_type *);
extern int x1f4_fini_qsrate(void **);
extern int x1f4_flat_qsrate(void *);
extern int x1f4_flip_qsrate(void *);
extern int x1f4_head_qsrate(void *, void **);
extern int x1f4_high_qsrate(void *);
extern int x1f4_init_qsrate(void **, unsigned, struct x1f4_qsrate_type *);
extern int x1f4_lead_qsrate(void *, void **);
extern int x1f4_lime_qsrate(void *, void *, int (*) (void *, void *));
extern int x1f4_line_qsrate
    (void *, void *, int (*) (void *, void *, const void *),
     int (*) (void *, void *), void *);
extern int x1f4_mind_qsrate(void *);
extern int x1f4_miss_qsrate(void *, unsigned);
extern int x1f4_over_qsrate
    (void *, unsigned, unsigned, void *, int (*) (void *, void *));
extern int x1f4_peek_qsrate(void *, unsigned, void **);
extern int x1f4_push_qsrate(void *, unsigned, void **);
extern int x1f4_rand_qsrate(void *);
extern int x1f4_size_qsrate(void *, unsigned *);
extern int x1f4_slip_qsrate(void *);
extern int x1f4_swap_qsrate(void *, void *);
extern int x1f4_tail_qsrate(void *, void **);
extern int x1f4_text_qsrate
    (void **, void *, int (*) (void *, void *, const void *),
     int (*) (void *, void *), void *);

#endif				/* __LIBx1f4_QSRATE_H__ */
